#ifndef _clip_h
#define _clip_h
#ifndef lint
static char rcsid_clip_h[] = "$Header: /net/hopkins/var/home/tristan/mpeg/dec/lib/include/RCS/clip.h,v 1.13 1997/10/31 02:12:16 tristan Exp $";
static char copyright_clip_h[] = "Copyright (c) 1997, MpegTV LLC";
#endif


/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

#include "mtvp/misc.h"

/*
 * This file defines optimized clamping macros:
 *
 * CLIP_0_255(x) -> clamp x in [0, 255]
 * CLIP_M128_127(x) -> clamp x in [-128, 127]
 * CLIP_0(x) -> clamp x in [0, +infinity]
 *
 * void clip_init(void) should be called before using any of those macros.
 *
 * Note: CLIP_SET() is now obsolete.
 */
 
/*--------------------------------------------------------------------------+
| Configuration flags														|
+--------------------------------------------------------------------------*/

#define CLIP_0_255_CACHE	/* use cache table for CLIP_0_255(x) */
/*#define CLIP_0_255_INLINE  use inline code rather than macro (when not cache) */

#define CLIP_M128_127_CACHE	/* use cache table rather than macho for CLIP_M128_127(x) */

/*
 * When we use lookup tables,
 * MAX_NEG_CLIP is the maximum neg/pos value we can handle.
 */
#define MAX_NEG_CLIP 2048

/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/

/* obsolete, keep it just for compatibitity */
#define CLIP_SET()

#ifdef CLIP_0_255_CACHE

#if 0 /* Tristan Savatier - Fri Feb 7 15:46:38 PST 1997 */
//#define CLIP_SET() 		u_char *_cm = Clip_table_0_255 + MAX_NEG_CLIP;
#endif /* 0 - Tristan Savatier - Fri Feb 7 15:46:38 PST 1997 */


#define CLIP_0_255(x) 	(Clip_table_0_255[(x) + MAX_NEG_CLIP])

#else /* not CLIP_0_255_CACHE */

#ifdef CLIP_0_255_INLINE

static INLINE int
CLIP_0_255(x)
	int	x;
{
	if ((((u_int)x) & ~0xff) != 0) {
		if (x<0) {
			return 0;
		} else {
			return 255;
		}
	} else {
		return x;
	}
}

#else /* not CLIP_0_255_INLINE */

#define CLIP_0_255(x) ((((u_int)(x)) & ~0xff) ? ((x) < 0 ? 0 : 255) : (x))

#endif /* not CLIP_0_255_INLINE */


#endif /* not CLIP_0_255_CACHE */

#ifdef CLIP_M128_127_CACHE

#define CLIP_M128_127(val)	Clip_table_M128_127[MAX_NEG_CLIP + (val)]

#else /* not CLIP_M128_127_CACHE */

#define CLIP_M128_127(val)	((val) < -128 ? -128 : (val) > 127 ? 127 : (val))

#endif /* not CLIP_M128_127_CACHE */


#if 0 /* Tristan Savatier - Sat Apr 27 03:24:04 PDT 1996 */
//static INLINE int
//CLIP_0_255(x)
//	int	x;
//{
//	if (x == (u_char)x) {
//		return x;
//	} else {
//		if (x<0) {
//			return 0;
//		} else {
//			return 255;
//		}
//	}
//}
#endif /* 0 */

#if 0 /* Tristan Savatier - Sat Apr 27 03:18:07 PDT 1996 */
//static INLINE u_char
//CLIP_0_255(x)
//	short	x;
//{
//	if (x & ~0xff) {
//		if (x<0) {
//			return 0;
//		} else {
//			return 255;
//		}
//	} else {
//		return x;
//	}
//}
#endif /* 0 */

#if 0 /* Tristan Savatier - Sat Apr 27 03:20:24 PDT 1996 */
//static INLINE u_char
//CLIP_0_255(x)
//	short	x;
//{
//	if ((((u_short)x)>>8) != 0) {
//		if (x<0) {
//			return 0;
//		} else {
//			return 255;
//		}
//	} else {
//		return x;
//	}
//}
#endif /* 0 */


#if 0 /* Tristan Savatier - Tue Dec 31 03:25:44 PST 1996 */
//#define CLIP_0(x)	((x) < 0 ? 0 : (x))
#endif /* 0 - Tristan Savatier - Tue Dec 31 03:25:44 PST 1996 */


static INLINE int
CLIP_0(x)
	int	x;
{
	if (x<0) {
		return 0;
	} else {
		return x;
	}
}

#if 0 /* Tristan Savatier - Fri Apr 26 03:40:34 PDT 1996 */
//static INLINE int
//clip(x, a, b)
//	int	x, a, b;
//{
//	if (x < a) {
//		return a;
//	} else if (x > b) {
//		return b;
//	} else {
//		return x;
//	}
//}
#endif /* 0 */

/*--------------------------------------------------------------------------+
| Externals																	|
+--------------------------------------------------------------------------*/
#ifdef CLIP_0_255_CACHE
	extern u_char Clip_table_0_255[];
#endif

#ifdef CLIP_M128_127_CACHE
	extern char Clip_table_M128_127[];
#endif

extern void clip_init(void);

#endif /* _clip_h */
